/*******************************************************************************
 * Copyright (c) 2009 itemis AG (http://www.itemis.eu) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.eclipse.emf.emfindex.store;

import java.io.Serializable;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.emfindex.BaseDescriptor;
import org.eclipse.emf.emfindex.ContainerDescriptor;
import org.eclipse.emf.emfindex.EObjectDescriptor;
import org.eclipse.emf.emfindex.EReferenceDescriptor;
import org.eclipse.emf.emfindex.ResourceDescriptor;
import org.eclipse.emf.index.ecore.emfindex.EmfindexFactory;
import org.eclipse.emf.index.ecore.emfindex.impl.AbstractBaseDescriptorImpl;
import org.eclipse.emf.index.ecore.emfindex.impl.ContainerDescriptorImpl;
import org.eclipse.emf.index.ecore.emfindex.impl.EObjectDescriptorImpl;
import org.eclipse.emf.index.ecore.emfindex.impl.EReferenceDescriptorImpl;
import org.eclipse.emf.index.ecore.emfindex.impl.ResourceDescriptorImpl;

/**
 * @author koehnlein
 */
public class DescriptorFactoryImpl implements DescriptorFactory {

	public ContainerDescriptor createContainerDescriptor(String name) {
		ContainerDescriptorImpl containerDescriptor = (ContainerDescriptorImpl) EmfindexFactory.eINSTANCE
				.createContainerDescriptor();
		containerDescriptor.setName(name);
		return containerDescriptor;
	}

	public ResourceDescriptor createResourceDescriptor(ContainerDescriptor container, URI resourceURI,
			long indexedVersion) {
		ResourceDescriptorImpl resourceDescriptor = (ResourceDescriptorImpl) EmfindexFactory.eINSTANCE
				.createResourceDescriptor();
		resourceDescriptor.setURI(resourceURI);
		resourceDescriptor.setIndexedVersion(indexedVersion);
		resourceDescriptor.setContainer(container);
		return resourceDescriptor;
	}

	public EObjectDescriptor createEObjectDescriptor(ResourceDescriptor container, String name,
			String fragment, EClass eClass) {
		EObjectDescriptorImpl eObjectDescriptor = (EObjectDescriptorImpl) EmfindexFactory.eINSTANCE
				.createEObjectDescriptor();
		eObjectDescriptor.setName(name);
		eObjectDescriptor.setFragment(fragment);
		eObjectDescriptor.setResourceDescriptor(container);
		eObjectDescriptor.setEClass(eClass);
		return eObjectDescriptor;
	}

	public EReferenceDescriptor createEReferenceDescriptor(ResourceDescriptor container, String sourceFragment,
			URI targetFragmentURI, EReference eReference) {
		EReferenceDescriptorImpl eReferenceDescriptor = (EReferenceDescriptorImpl) EmfindexFactory.eINSTANCE
				.createEReferenceDescriptor();
		eReferenceDescriptor.setSourceFragment(sourceFragment);
		eReferenceDescriptor.setTargetFragment(targetFragmentURI.fragment());
		eReferenceDescriptor.setTargetResourceURI(targetFragmentURI.trimFragment());
		eReferenceDescriptor.setEReference(eReference);
		eReferenceDescriptor.setSourceResourceDescriptor(container);
		return eReferenceDescriptor;
	}

	public EReferenceDescriptor createEReferenceDescriptor(ResourceDescriptor container, String sourceFragment,
			EObjectDescriptor targetDescriptor, EReference eReference) {
		EReferenceDescriptorImpl eReferenceDescriptor = (EReferenceDescriptorImpl) EmfindexFactory.eINSTANCE
				.createEReferenceDescriptor();
		eReferenceDescriptor.setSourceFragment(sourceFragment);
		eReferenceDescriptor.setTargetEObjectDescriptor(targetDescriptor);
		eReferenceDescriptor.setEReference(eReference);
		eReferenceDescriptor.setSourceResourceDescriptor(container);
		return eReferenceDescriptor;
	}

	public void addUserData(BaseDescriptor descriptor, String key, Serializable value) {
		AbstractBaseDescriptorImpl emfDescriptor = (AbstractBaseDescriptorImpl) descriptor;
		emfDescriptor.setUserData(key, value);
	}

}
